@@ -1,4 +1,5 @@
 """All pytest-django fixtures"""
+
 from __future__ import annotations
 
 import os
@@ -6,9 +7,9 @@ from contextlib import contextmanager
 from functools import partial
 from typing import (
     TYPE_CHECKING,
+    AbstractSet,
     Any,
     Callable,
-    ClassVar,
     ContextManager,
     Generator,
     Iterable,
@@ -16,6 +17,7 @@ from typing import (
     Literal,
     Optional,
     Protocol,
+    Sequence,
     Tuple,
     Union,
 )
@@ -41,25 +43,25 @@ _DjangoDb = Tuple[bool, bool, _DjangoDbDatabases, bool, _DjangoDbAvailableApps]
 
 
 __all__ = [
-    "django_db_setup",
+    "_live_server_helper",
+    "admin_client",
+    "admin_user",
+    "async_client",
+    "async_rf",
+    "client",
     "db",
-    "transactional_db",
+    "django_assert_max_num_queries",
+    "django_assert_num_queries",
+    "django_capture_on_commit_callbacks",
     "django_db_reset_sequences",
     "django_db_serialized_rollback",
-    "admin_user",
+    "django_db_setup",
     "django_user_model",
     "django_username_field",
-    "client",
-    "async_client",
-    "admin_client",
-    "rf",
-    "async_rf",
-    "settings",
     "live_server",
-    "_live_server_helper",
-    "django_assert_num_queries",
-    "django_assert_max_num_queries",
-    "django_capture_on_commit_callbacks",
+    "rf",
+    "settings",
+    "transactional_db",
 ]
 
 
@@ -95,26 +97,80 @@ def django_db_modify_db_settings_parallel_suffix(
 def django_db_modify_db_settings(
     django_db_modify_db_settings_parallel_suffix: None,
 ) -> None:
+    """Modify db settings just before the databases are configured."""
     skip_if_no_django()
 
 
 @pytest.fixture(scope="session")
 def django_db_use_migrations(request: pytest.FixtureRequest) -> bool:
+    """Return whether to use migrations to create the test databases."""
     return not request.config.getvalue("nomigrations")
 
 
 @pytest.fixture(scope="session")
 def django_db_keepdb(request: pytest.FixtureRequest) -> bool:
+    """Return whether to re-use an existing database and to keep it after the test run."""
     reuse_db: bool = request.config.getvalue("reuse_db")
     return reuse_db
 
 
 @pytest.fixture(scope="session")
 def django_db_createdb(request: pytest.FixtureRequest) -> bool:
+    """Return whether the database is to be re-created before running any tests."""
     create_db: bool = request.config.getvalue("create_db")
     return create_db
 
 
+def _get_databases_for_test(test: pytest.Item) -> tuple[Iterable[str], bool]:
+    """Get the database aliases that need to be setup for a test, and whether
+    they need to be serialized."""
+    from django.db import DEFAULT_DB_ALIAS, connections
+    from django.test import TransactionTestCase
+
+    test_cls = getattr(test, "cls", None)
+    if test_cls and issubclass(test_cls, TransactionTestCase):
+        serialized_rollback = getattr(test, "serialized_rollback", False)
+        databases = getattr(test, "databases", None)
+    else:
+        fixtures = getattr(test, "fixturenames", ())
+        marker_db = test.get_closest_marker("django_db")
+        if marker_db:
+            (
+                transaction,
+                reset_sequences,
+                databases,
+                serialized_rollback,
+                available_apps,
+            ) = validate_django_db(marker_db)
+        elif "db" in fixtures or "transactional_db" in fixtures or "live_server" in fixtures:
+            serialized_rollback = "django_db_serialized_rollback" in fixtures
+            databases = None
+        else:
+            return (), False
+    if databases is None:
+        return (DEFAULT_DB_ALIAS,), serialized_rollback
+    elif databases == "__all__":
+        return connections, serialized_rollback
+    else:
+        return databases, serialized_rollback
+
+
+def _get_databases_for_setup(
+    items: Sequence[pytest.Item],
+) -> tuple[AbstractSet[str], AbstractSet[str]]:
+    """Get the database aliases that need to be setup, and the subset that needs
+    to be serialized."""
+    # Code derived from django.test.utils.DiscoverRunner.get_databases().
+    aliases: set[str] = set()
+    serialized_aliases: set[str] = set()
+    for test in items:
+        databases, serialized_rollback = _get_databases_for_test(test)
+        aliases.update(databases)
+        if serialized_rollback:
+            serialized_aliases.update(databases)
+    return aliases, serialized_aliases
+
+
 @pytest.fixture(scope="session")
 def django_db_setup(
     request: pytest.FixtureRequest,
@@ -136,10 +192,14 @@ def django_db_setup(
     if django_db_keepdb and not django_db_createdb:
         setup_databases_args["keepdb"] = True
 
+    aliases, serialized_aliases = _get_databases_for_setup(request.session.items)
+
     with django_db_blocker.unblock():
         db_cfg = setup_databases(
             verbosity=request.config.option.verbose,
             interactive=False,
+            aliases=aliases,
+            serialized_aliases=serialized_aliases,
             **setup_databases_args,
         )
 
@@ -161,8 +221,6 @@ def _django_db_helper(
     django_db_setup: None,
     django_db_blocker: DjangoDbBlocker,
 ) -> Generator[None, None, None]:
-    from django import VERSION
-
     if is_django_unittest(request):
         yield
         return
@@ -195,74 +253,77 @@ def _django_db_helper(
         "django_db_serialized_rollback" in request.fixturenames
     )
 
-    django_db_blocker.unblock()
+    with django_db_blocker.unblock():
+        import django.db
+        import django.test
 
-    import django.db
-    import django.test
+        if transactional:
+            test_case_class = django.test.TransactionTestCase
+        else:
+            test_case_class = django.test.TestCase
 
-    if transactional:
-        test_case_class = django.test.TransactionTestCase
-    else:
-        test_case_class = django.test.TestCase
+        _reset_sequences = reset_sequences
+        _serialized_rollback = serialized_rollback
+        _databases = databases
+        _available_apps = available_apps
 
-    _reset_sequences = reset_sequences
-    _serialized_rollback = serialized_rollback
-    _databases = databases
-    _available_apps = available_apps
+        class PytestDjangoTestCase(test_case_class):  # type: ignore[misc,valid-type]
+            reset_sequences = _reset_sequences
+            serialized_rollback = _serialized_rollback
+            if _databases is not None:
+                databases = _databases
+            if _available_apps is not None:
+                available_apps = _available_apps
 
-    class PytestDjangoTestCase(test_case_class):  # type: ignore[misc,valid-type]
-        reset_sequences = _reset_sequences
-        serialized_rollback = _serialized_rollback
-        if _databases is not None:
-            databases = _databases
-        if _available_apps is not None:
-            available_apps = _available_apps
+            # For non-transactional tests, skip executing `django.test.TestCase`'s
+            # `setUpClass`/`tearDownClass`, only execute the super class ones.
+            #
+            # `TestCase`'s class setup manages the `setUpTestData`/class-level
+            # transaction functionality. We don't use it; instead we (will) offer
+            # our own alternatives. So it only adds overhead, and does some things
+            # which conflict with our (planned) functionality, particularly, it
+            # closes all database connections in `tearDownClass` which inhibits
+            # wrapping tests in higher-scoped transactions.
+            #
+            # It's possible a new version of Django will add some unrelated
+            # functionality to these methods, in which case skipping them completely
+            # would not be desirable. Let's cross that bridge when we get there...
+            if not transactional:
 
-        # For non-transactional tests, skip executing `django.test.TestCase`'s
-        # `setUpClass`/`tearDownClass`, only execute the super class ones.
-        #
-        # `TestCase`'s class setup manages the `setUpTestData`/class-level
-        # transaction functionality. We don't use it; instead we (will) offer
-        # our own alternatives. So it only adds overhead, and does some things
-        # which conflict with our (planned) functionality, particularly, it
-        # closes all database connections in `tearDownClass` which inhibits
-        # wrapping tests in higher-scoped transactions.
-        #
-        # It's possible a new version of Django will add some unrelated
-        # functionality to these methods, in which case skipping them completely
-        # would not be desirable. Let's cross that bridge when we get there...
-        if not transactional:
+                @classmethod
+                def setUpClass(cls) -> None:
+                    super(django.test.TestCase, cls).setUpClass()
 
-            @classmethod
-            def setUpClass(cls) -> None:
-                super(django.test.TestCase, cls).setUpClass()
-                if VERSION < (4, 1):
-                    django.db.transaction.Atomic._ensure_durability = False
+                @classmethod
+                def tearDownClass(cls) -> None:
+                    super(django.test.TestCase, cls).tearDownClass()
 
-            @classmethod
-            def tearDownClass(cls) -> None:
-                if VERSION < (4, 1):
-                    django.db.transaction.Atomic._ensure_durability = True
-                super(django.test.TestCase, cls).tearDownClass()
+        PytestDjangoTestCase.setUpClass()
 
-    PytestDjangoTestCase.setUpClass()
+        test_case = PytestDjangoTestCase(methodName="__init__")
+        test_case._pre_setup()
 
-    test_case = PytestDjangoTestCase(methodName="__init__")
-    test_case._pre_setup()
+        yield
 
-    yield
+        test_case._post_teardown()
 
-    test_case._post_teardown()
+        PytestDjangoTestCase.tearDownClass()
 
-    PytestDjangoTestCase.tearDownClass()
-
-    if VERSION >= (4, 0):
         PytestDjangoTestCase.doClassCleanups()
 
-    django_db_blocker.restore()
+
+def _django_db_signature(
+    transaction: bool = False,
+    reset_sequences: bool = False,
+    databases: _DjangoDbDatabases = None,
+    serialized_rollback: bool = False,
+    available_apps: _DjangoDbAvailableApps = None,
+) -> _DjangoDb:
+    """The signature of the django_db marker. Used by validate_django_db."""
+    return transaction, reset_sequences, databases, serialized_rollback, available_apps
 
 
-def validate_django_db(marker) -> _DjangoDb:
+def validate_django_db(marker: pytest.Mark) -> _DjangoDb:
     """Validate the django_db marker.
 
     It checks the signature and creates the ``transaction``,
@@ -273,17 +334,7 @@ def validate_django_db(marker) -> _DjangoDb:
     Sequence reset, serialized_rollback, and available_apps are only allowed
     when combined with transaction.
     """
-
-    def apifun(
-        transaction: bool = False,
-        reset_sequences: bool = False,
-        databases: _DjangoDbDatabases = None,
-        serialized_rollback: bool = False,
-        available_apps: _DjangoDbAvailableApps = None,
-    ) -> _DjangoDb:
-        return transaction, reset_sequences, databases, serialized_rollback, available_apps
-
-    return apifun(*marker.args, **marker.kwargs)
+    return _django_db_signature(*marker.args, **marker.kwargs)
 
 
 def _disable_migrations() -> None:
@@ -498,7 +549,9 @@ def async_rf() -> django.test.AsyncRequestFactory:
 
 
 class SettingsWrapper:
-    _to_restore: ClassVar[list[Any]] = []
+    def __init__(self) -> None:
+        self._to_restore: list[django.test.override_settings]
+        object.__setattr__(self, "_to_restore", [])
 
     def __delattr__(self, attr: str) -> None:
         from django.test import override_settings
@@ -610,6 +663,8 @@ class DjangoAssertNumQueries(Protocol):
         num: int,
         connection: Any | None = ...,
         info: str | None = ...,
+        *,
+        using: str | None = ...,
     ) -> django.test.utils.CaptureQueriesContext:
         pass  # pragma: no cover
 
@@ -621,13 +676,21 @@ def _assert_num_queries(
     exact: bool = True,
     connection: Any | None = None,
     info: str | None = None,
+    *,
+    using: str | None = None,
 ) -> Generator[django.test.utils.CaptureQueriesContext, None, None]:
+    from django.db import connection as default_conn, connections
     from django.test.utils import CaptureQueriesContext
 
-    if connection is None:
-        from django.db import connection as conn
-    else:
+    if connection and using:
+        raise ValueError('The "connection" and "using" parameter cannot be used together')
+
+    if connection is not None:
         conn = connection
+    elif using is not None:
+        conn = connections[using]
+    else:
+        conn = default_conn
 
     verbose = config.getoption("verbose") > 0
     with CaptureQueriesContext(conn) as context:
@@ -655,11 +718,13 @@ def _assert_num_queries(
 
 @pytest.fixture()
 def django_assert_num_queries(pytestconfig: pytest.Config) -> DjangoAssertNumQueries:
+    """Allows to check for an expected number of DB queries."""
     return partial(_assert_num_queries, pytestconfig)
 
 
 @pytest.fixture()
 def django_assert_max_num_queries(pytestconfig: pytest.Config) -> DjangoAssertNumQueries:
+    """Allows to check for an expected maximum number of DB queries."""
     return partial(_assert_num_queries, pytestconfig, exact=False)
 
 
@@ -677,6 +742,7 @@ class DjangoCaptureOnCommitCallbacks(Protocol):
 
 @pytest.fixture()
 def django_capture_on_commit_callbacks() -> DjangoCaptureOnCommitCallbacks:
+    """Captures transaction.on_commit() callbacks for the given database connection."""
     from django.test import TestCase
 
     return TestCase.captureOnCommitCallbacks  # type: ignore[no-any-return]
